"""Meshtastic smoke tests with a single virtual device via localhost.

   During the CI build of the Meshtastic-device, a build.zip file is created.
   Inside that build.zip is a standalone executable meshtasticd_linux_amd64.
   That linux executable will simulate a Meshtastic device listening on localhost.

   This smoke test runs against that localhost.

"""
import os
import platform
import re
import subprocess
import time

# Do not like using hard coded sleeps, but it probably makes
# sense to pause for the radio at appropriate times
import pytest

from ..util import findPorts

# seconds to pause after running a meshtastic command
PAUSE_AFTER_COMMAND = 0.1
PAUSE_AFTER_REBOOT = 0.2


# TODO: need to fix the virtual device to have a reboot. When you issue the command
#      below, you get "FIXME implement reboot for this platform"
# @pytest.mark.smokevirt
# def test_smokevirt_reboot():
#    """Test reboot"""
#    return_value, _ = subprocess.getstatusoutput('meshtastic --host localhost --reboot')
#    assert return_value == 0
#    # pause for the radio to reset
#    time.sleep(8)


@pytest.mark.smokevirt
def test_smokevirt_info():
    """Test --info"""
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.match(r"Connected to radio", out)
    assert re.search(r"^Owner", out, re.MULTILINE)
    assert re.search(r"^My info", out, re.MULTILINE)
    assert re.search(r"^Nodes in mesh", out, re.MULTILINE)
    assert re.search(r"^Preferences", out, re.MULTILINE)
    assert re.search(r"^Channels", out, re.MULTILINE)
    assert re.search(r"^  PRIMARY", out, re.MULTILINE)
    assert re.search(r"^Primary channel URL", out, re.MULTILINE)
    assert return_value == 0


@pytest.mark.smokevirt
def test_get_with_invalid_setting():
    """Test '--get a_bad_setting'."""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --get a_bad_setting"
    )
    assert re.search(r"Choices in sorted order", out)
    assert return_value == 0


@pytest.mark.smokevirt
def test_set_with_invalid_setting():
    """Test '--set a_bad_setting'."""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --set a_bad_setting foo"
    )
    assert re.search(r"Choices in sorted order", out)
    assert return_value == 0


@pytest.mark.smokevirt
def test_ch_set_with_invalid_settingpatch_find_ports():
    """Test '--ch-set with a_bad_setting'."""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-set invalid_setting foo --ch-index 0"
    )
    assert re.search(r"Choices in sorted order", out)
    assert return_value == 0


@pytest.mark.smokevirt
def test_smokevirt_pos_fields():
    """Test --pos-fields (with some values POS_ALTITUDE POS_ALT_MSL POS_BATTERY)"""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --pos-fields POS_ALTITUDE POS_ALT_MSL POS_BATTERY"
    )
    assert re.match(r"Connected to radio", out)
    assert re.search(r"^Setting position fields to 35", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --pos-fields"
    )
    assert re.match(r"Connected to radio", out)
    assert re.search(r"POS_ALTITUDE", out, re.MULTILINE)
    assert re.search(r"POS_ALT_MSL", out, re.MULTILINE)
    assert re.search(r"POS_BATTERY", out, re.MULTILINE)
    assert return_value == 0


@pytest.mark.smokevirt
def test_smokevirt_test_with_arg_but_no_hardware():
    """Test --test
    Note: Since only one device is connected, it will not do much.
    """
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --test")
    assert re.search(r"^Warning: Must have at least two devices", out, re.MULTILINE)
    assert return_value == 1


@pytest.mark.smokevirt
def test_smokevirt_debug():
    """Test --debug"""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --info --debug"
    )
    assert re.search(r"^Owner", out, re.MULTILINE)
    assert re.search(r"^DEBUG file", out, re.MULTILINE)
    assert return_value == 0


@pytest.mark.smokevirt
def test_smokevirt_seriallog_to_file():
    """Test --seriallog to a file creates a file"""
    filename = "tmpoutput.txt"
    if os.path.exists(f"{filename}"):
        os.remove(f"{filename}")
    return_value, _ = subprocess.getstatusoutput(
        f"meshtastic --host localhost --info --seriallog {filename}"
    )
    assert os.path.exists(f"{filename}")
    assert return_value == 0
    os.remove(f"{filename}")


@pytest.mark.smokevirt
def test_smokevirt_qr():
    """Test --qr"""
    filename = "tmpqr"
    if os.path.exists(f"{filename}"):
        os.remove(f"{filename}")
    return_value, _ = subprocess.getstatusoutput(
        f"meshtastic --host localhost --qr > {filename}"
    )
    assert os.path.exists(f"{filename}")
    # not really testing that a valid qr code is created, just that the file size
    # is reasonably big enough for a qr code
    assert os.stat(f"{filename}").st_size > 20000
    assert return_value == 0
    os.remove(f"{filename}")


@pytest.mark.smokevirt
def test_smokevirt_nodes():
    """Test --nodes"""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --nodes"
    )
    assert re.match(r"Connected to radio", out)
    if platform.system() != "Windows":
        assert re.search(r" User ", out, re.MULTILINE)
        assert re.search(r"  1 ", out, re.MULTILINE)
    assert return_value == 0


@pytest.mark.smokevirt
def test_smokevirt_send_hello():
    """Test --sendtext hello"""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --sendtext hello"
    )
    assert re.match(r"Connected to radio", out)
    assert re.search(r"^Sending text message hello to \^all", out, re.MULTILINE)
    assert return_value == 0


@pytest.mark.smokevirt
def test_smokevirt_port():
    """Test --port"""
    # first, get the ports
    ports = findPorts()
    # hopefully there is none
    assert len(ports) == 0


@pytest.mark.smokevirt
def test_smokevirt_set_location_info():
    """Test --setlat, --setlon and --setalt"""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --setlat 32.7767 --setlon -96.7970 --setalt 1337"
    )
    assert re.match(r"Connected to radio", out)
    assert re.search(r"^Fixing altitude", out, re.MULTILINE)
    assert re.search(r"^Fixing latitude", out, re.MULTILINE)
    assert re.search(r"^Fixing longitude", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out2 = subprocess.getstatusoutput(
        "meshtastic --host localhost --info"
    )
    assert re.search(r"1337", out2, re.MULTILINE)
    assert re.search(r"32.7767", out2, re.MULTILINE)
    assert re.search(r"-96.797", out2, re.MULTILINE)
    assert return_value == 0


@pytest.mark.smokevirt
def test_smokevirt_set_owner():
    """Test --set-owner name"""
    # make sure the owner is not Joe
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --set-owner Bob"
    )
    assert re.match(r"Connected to radio", out)
    assert re.search(r"^Setting device owner to Bob", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert not re.search(r"Owner: Joe", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --set-owner Joe"
    )
    assert re.match(r"Connected to radio", out)
    assert re.search(r"^Setting device owner to Joe", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.search(r"Owner: Joe", out, re.MULTILINE)
    assert return_value == 0


@pytest.mark.smokevirt
def test_smokevirt_ch_values():
    """Test --ch-longslow, --ch-longfast, --ch-mediumslow, --ch-mediumsfast,
    --ch-shortslow, and --ch-shortfast arguments
    """
    exp = {
        "--ch-longslow": "LongSlow",
        "--ch-longfast": "LongFast",
        "--ch-medslow": "MedSlow",
        "--ch-medfast": "MedFast",
        "--ch-shortslow": "ShortSlow",
        "--ch-shortfast": "ShortFast",
    }

    for key, val in exp.items():
        return_value, out = subprocess.getstatusoutput(
            f"meshtastic --host localhost {key}"
        )
        assert re.match(r"Connected to radio", out)
        assert re.search(r"Writing modified channels to device", out, re.MULTILINE)
        assert return_value == 0
        # pause for the radio (might reboot)
        time.sleep(PAUSE_AFTER_REBOOT)
        return_value, out = subprocess.getstatusoutput(
            "meshtastic --host localhost --info"
        )
        assert re.search(val, out, re.MULTILINE)
        assert return_value == 0
        # pause for the radio
        time.sleep(PAUSE_AFTER_COMMAND)


@pytest.mark.smokevirt
def test_smokevirt_ch_set_name():
    """Test --ch-set name"""
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert not re.search(r"MyChannel", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-set name MyChannel"
    )
    assert re.match(r"Connected to radio", out)
    assert re.search(r"Warning: Need to specify", out, re.MULTILINE)
    assert return_value == 1
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-set name MyChannel --ch-index 0"
    )
    assert re.match(r"Connected to radio", out)
    assert re.search(r"^Set name to MyChannel", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.search(r"MyChannel", out, re.MULTILINE)
    assert return_value == 0


@pytest.mark.smokevirt
def test_smokevirt_ch_set_downlink_and_uplink():
    """Test -ch-set downlink_enabled X and --ch-set uplink_enabled X"""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-set downlink_enabled false --ch-set uplink_enabled false"
    )
    assert re.match(r"Connected to radio", out)
    assert re.search(r"Warning: Need to specify", out, re.MULTILINE)
    assert return_value == 1
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    # pylint: disable=C0301
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-set downlink_enabled false --ch-set uplink_enabled false --ch-index 0"
    )
    assert re.match(r"Connected to radio", out)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert not re.search(r"uplinkEnabled", out, re.MULTILINE)
    assert not re.search(r"downlinkEnabled", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    # pylint: disable=C0301
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-set downlink_enabled true --ch-set uplink_enabled true --ch-index 0"
    )
    assert re.match(r"Connected to radio", out)
    assert re.search(r"^Set downlink_enabled to true", out, re.MULTILINE)
    assert re.search(r"^Set uplink_enabled to true", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.search(r"uplinkEnabled", out, re.MULTILINE)
    assert re.search(r"downlinkEnabled", out, re.MULTILINE)
    assert return_value == 0


@pytest.mark.smokevirt
def test_smokevirt_ch_add_and_ch_del():
    """Test --ch-add"""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-index 1 --ch-del"
    )
    assert re.search(r"Deleting channel 1", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_REBOOT)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-add testing"
    )
    assert re.search(r"Writing modified channels to device", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.match(r"Connected to radio", out)
    assert re.search(r"SECONDARY", out, re.MULTILINE)
    assert re.search(r"testing", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-index 1 --ch-del"
    )
    assert re.search(r"Deleting channel 1", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_REBOOT)
    # make sure the secondary channel is not there
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.match(r"Connected to radio", out)
    assert not re.search(r"SECONDARY", out, re.MULTILINE)
    assert not re.search(r"testing", out, re.MULTILINE)
    assert return_value == 0


@pytest.mark.smokevirt
def test_smokevirt_ch_enable_and_disable():
    """Test --ch-enable and --ch-disable"""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-index 1 --ch-del"
    )
    assert re.search(r"Deleting channel 1", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_REBOOT)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-add testing"
    )
    assert re.search(r"Writing modified channels to device", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.match(r"Connected to radio", out)
    assert re.search(r"SECONDARY", out, re.MULTILINE)
    assert re.search(r"testing", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    # ensure they need to specify a --ch-index
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-disable"
    )
    assert return_value == 1
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-disable --ch-index 1"
    )
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.match(r"Connected to radio", out)
    assert re.search(r"DISABLED", out, re.MULTILINE)
    assert re.search(r"testing", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-enable --ch-index 1"
    )
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.match(r"Connected to radio", out)
    assert re.search(r"SECONDARY", out, re.MULTILINE)
    assert re.search(r"testing", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-del --ch-index 1"
    )
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)


@pytest.mark.smokevirt
def test_smokevirt_ch_del_a_disabled_non_primary_channel():
    """Test --ch-del will work on a disabled non-primary channel."""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-index 1 --ch-del"
    )
    assert re.search(r"Deleting channel 1", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_REBOOT)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-add testing"
    )
    assert re.search(r"Writing modified channels to device", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.match(r"Connected to radio", out)
    assert re.search(r"SECONDARY", out, re.MULTILINE)
    assert re.search(r"testing", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    # ensure they need to specify a --ch-index
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-disable"
    )
    assert return_value == 1
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-del --ch-index 1"
    )
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.match(r"Connected to radio", out)
    assert not re.search(r"DISABLED", out, re.MULTILINE)
    assert not re.search(r"SECONDARY", out, re.MULTILINE)
    assert not re.search(r"testing", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)


@pytest.mark.smokevirt
def test_smokevirt_attempt_to_delete_primary_channel():
    """Test that we cannot delete the PRIMARY channel."""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-del --ch-index 0"
    )
    assert re.search(r"Warning: Cannot delete primary channel", out, re.MULTILINE)
    assert return_value == 1
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)


@pytest.mark.smokevirt
def test_smokevirt_attempt_to_disable_primary_channel():
    """Test that we cannot disable the PRIMARY channel."""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-disable --ch-index 0"
    )
    assert re.search(r"Warning: Cannot enable", out, re.MULTILINE)
    assert return_value == 1
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)


@pytest.mark.smokevirt
def test_smokevirt_attempt_to_enable_primary_channel():
    """Test that we cannot enable the PRIMARY channel."""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-enable --ch-index 0"
    )
    assert re.search(r"Warning: Cannot enable", out, re.MULTILINE)
    assert return_value == 1
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)


@pytest.mark.smokevirt
def test_smokevirt_ensure_ch_del_second_of_three_channels():
    """Test that when we delete the 2nd of 3 channels, that it deletes the correct channel."""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-add testing1"
    )
    assert re.match(r"Connected to radio", out)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.match(r"Connected to radio", out)
    assert re.search(r"SECONDARY", out, re.MULTILINE)
    assert re.search(r"testing1", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-add testing2"
    )
    assert re.match(r"Connected to radio", out)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.match(r"Connected to radio", out)
    assert re.search(r"testing2", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-del --ch-index 1"
    )
    assert re.match(r"Connected to radio", out)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.match(r"Connected to radio", out)
    assert re.search(r"testing2", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-del --ch-index 1"
    )
    assert re.match(r"Connected to radio", out)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)


@pytest.mark.smokevirt
def test_smokevirt_ensure_ch_del_third_of_three_channels():
    """Test that when we delete the 3rd of 3 channels, that it deletes the correct channel."""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-add testing1"
    )
    assert re.match(r"Connected to radio", out)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.match(r"Connected to radio", out)
    assert re.search(r"SECONDARY", out, re.MULTILINE)
    assert re.search(r"testing1", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-add testing2"
    )
    assert re.match(r"Connected to radio", out)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.match(r"Connected to radio", out)
    assert re.search(r"testing2", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-del --ch-index 2"
    )
    assert re.match(r"Connected to radio", out)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.match(r"Connected to radio", out)
    assert re.search(r"testing1", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-del --ch-index 1"
    )
    assert re.match(r"Connected to radio", out)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)


@pytest.mark.smokevirt
def test_smokevirt_ch_set_modem_config():
    """Test --ch-set modem_config"""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-set modem_config Bw31_25Cr48Sf512"
    )
    assert re.search(r"Warning: Need to specify", out, re.MULTILINE)
    assert return_value == 1
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert not re.search(r"Bw31_25Cr48Sf512", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-set modem_config MidSlow --ch-index 0"
    )
    assert re.match(r"Connected to radio", out)
    assert re.search(r"^Set modem_config to MidSlow", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.search(r"MidSlow", out, re.MULTILINE)
    assert return_value == 0


@pytest.mark.smokevirt
def test_smokevirt_seturl_default():
    """Test --seturl with default value"""
    # set some channel value so we no longer have a default channel
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --ch-set name foo --ch-index 0"
    )
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    # ensure we no longer have a default primary channel
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert not re.search("CgUYAyIBAQ", out, re.MULTILINE)
    assert return_value == 0
    url = "https://www.meshtastic.org/d/#CgUYAyIBAQ"
    return_value, out = subprocess.getstatusoutput(
        f"meshtastic --host localhost --seturl {url}"
    )
    assert re.match(r"Connected to radio", out)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.search("CgUYAyIBAQ", out, re.MULTILINE)
    assert return_value == 0


@pytest.mark.smokevirt
def test_smokevirt_seturl_invalid_url():
    """Test --seturl with invalid url"""
    # Note: This url is no longer a valid url.
    url = "https://www.meshtastic.org/c/#GAMiENTxuzogKQdZ8Lz_q89Oab8qB0RlZmF1bHQ="
    return_value, out = subprocess.getstatusoutput(
        f"meshtastic --host localhost --seturl {url}"
    )
    assert re.match(r"Connected to radio", out)
    assert re.search("Warning: There were no settings", out, re.MULTILINE)
    assert return_value == 1
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)


@pytest.mark.smokevirt
def test_smokevirt_configure():
    """Test --configure"""
    _, out = subprocess.getstatusoutput(
        f"meshtastic --host localhost --configure example_config.yaml"
    )
    assert re.match(r"Connected to radio", out)
    assert re.search("^Setting device owner to Bob TBeam", out, re.MULTILINE)
    assert re.search("^Fixing altitude at 304 meters", out, re.MULTILINE)
    assert re.search("^Fixing latitude at 35.8", out, re.MULTILINE)
    assert re.search("^Fixing longitude at -93.8", out, re.MULTILINE)
    assert re.search("^Setting device position", out, re.MULTILINE)
    assert re.search("^Set region to 1", out, re.MULTILINE)
    assert re.search("^Set is_always_powered to true", out, re.MULTILINE)
    assert re.search("^Set send_owner_interval to 2", out, re.MULTILINE)
    assert re.search("^Set screen_on_secs to 31536000", out, re.MULTILINE)
    assert re.search("^Set wait_bluetooth_secs to 31536000", out, re.MULTILINE)
    assert re.search("^Writing modified preferences to device", out, re.MULTILINE)
    # pause for the radio
    time.sleep(PAUSE_AFTER_REBOOT)


@pytest.mark.smokevirt
def test_smokevirt_set_ham():
    """Test --set-ham
    Note: Do a factory reset after this setting so it is very short-lived.
    """
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --set-ham KI1234"
    )
    assert re.search(r"Setting Ham ID", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_REBOOT)
    return_value, out = subprocess.getstatusoutput("meshtastic --host localhost --info")
    assert re.search(r"Owner: KI1234", out, re.MULTILINE)
    assert return_value == 0


@pytest.mark.smokevirt
def test_smokevirt_set_wifi_settings():
    """Test --set wifi_ssid and --set wifi_password"""
    return_value, out = subprocess.getstatusoutput(
        'meshtastic --host localhost --set wifi_ssid "some_ssid" --set wifi_password "temp1234"'
    )
    assert re.match(r"Connected to radio", out)
    assert re.search(r"^Set wifi_ssid to some_ssid", out, re.MULTILINE)
    assert re.search(r"^Set wifi_password to temp1234", out, re.MULTILINE)
    assert return_value == 0
    # pause for the radio
    time.sleep(PAUSE_AFTER_COMMAND)
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --get wifi_ssid --get wifi_password"
    )
    assert re.search(r"^wifi_ssid: some_ssid", out, re.MULTILINE)
    assert re.search(r"^wifi_password: sekrit", out, re.MULTILINE)
    assert return_value == 0


@pytest.mark.smokevirt
def test_smokevirt_factory_reset():
    """Test factory reset"""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --host localhost --set factory_reset true"
    )
    assert re.match(r"Connected to radio", out)
    assert re.search(r"^Set factory_reset to true", out, re.MULTILINE)
    assert re.search(r"^Writing modified preferences to device", out, re.MULTILINE)
    assert return_value == 0
    # NOTE: The virtual radio will not respond well after this command. Need to re-start the virtual program at this point.
    # TODO: fix?



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: meshtastic/protobuf/portnums.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"meshtastic/protobuf/portnums.proto\x12\x13meshtastic.protobuf*\xb1\x04\n\x07PortNum\x12\x0f\n\x0bUNKNOWN_APP\x10\x00\x12\x14\n\x10TEXT_MESSAGE_APP\x10\x01\x12\x17\n\x13REMOTE_HARDWARE_APP\x10\x02\x12\x10\n\x0cPOSITION_APP\x10\x03\x12\x10\n\x0cNODEINFO_APP\x10\x04\x12\x0f\n\x0bROUTING_APP\x10\x05\x12\r\n\tADMIN_APP\x10\x06\x12\x1f\n\x1bTEXT_MESSAGE_COMPRESSED_APP\x10\x07\x12\x10\n\x0cWAYPOINT_APP\x10\x08\x12\r\n\tAUDIO_APP\x10\t\x12\x18\n\x14\x44\x45TECTION_SENSOR_APP\x10\n\x12\r\n\tALERT_APP\x10\x0b\x12\r\n\tREPLY_APP\x10 \x12\x11\n\rIP_TUNNEL_APP\x10!\x12\x12\n\x0ePAXCOUNTER_APP\x10\"\x12\x0e\n\nSERIAL_APP\x10@\x12\x15\n\x11STORE_FORWARD_APP\x10\x41\x12\x12\n\x0eRANGE_TEST_APP\x10\x42\x12\x11\n\rTELEMETRY_APP\x10\x43\x12\x0b\n\x07ZPS_APP\x10\x44\x12\x11\n\rSIMULATOR_APP\x10\x45\x12\x12\n\x0eTRACEROUTE_APP\x10\x46\x12\x14\n\x10NEIGHBORINFO_APP\x10G\x12\x0f\n\x0b\x41TAK_PLUGIN\x10H\x12\x12\n\x0eMAP_REPORT_APP\x10I\x12\x13\n\x0fPOWERSTRESS_APP\x10J\x12\x10\n\x0bPRIVATE_APP\x10\x80\x02\x12\x13\n\x0e\x41TAK_FORWARDER\x10\x81\x02\x12\x08\n\x03MAX\x10\xff\x03\x42]\n\x13\x63om.geeksville.meshB\x08PortnumsZ\"github.com/meshtastic/go/generated\xaa\x02\x14Meshtastic.Protobufs\xba\x02\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meshtastic.protobuf.portnums_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.geeksville.meshB\010PortnumsZ\"github.com/meshtastic/go/generated\252\002\024Meshtastic.Protobufs\272\002\000'
  _globals['_PORTNUM']._serialized_start=60
  _globals['_PORTNUM']._serialized_end=621
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _PortNum:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _PortNumEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_PortNum.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN_APP: _PortNum.ValueType  # 0
    """
    Deprecated: do not use in new code (formerly called OPAQUE)
    A message sent from a device outside of the mesh, in a form the mesh does not understand
    NOTE: This must be 0, because it is documented in IMeshService.aidl to be so
    ENCODING: binary undefined
    """
    TEXT_MESSAGE_APP: _PortNum.ValueType  # 1
    """
    A simple UTF-8 text message, which even the little micros in the mesh
    can understand and show on their screen eventually in some circumstances
    even signal might send messages in this form (see below)
    ENCODING: UTF-8 Plaintext (?)
    """
    REMOTE_HARDWARE_APP: _PortNum.ValueType  # 2
    """
    Reserved for built-in GPIO/example app.
    See remote_hardware.proto/HardwareMessage for details on the message sent/received to this port number
    ENCODING: Protobuf
    """
    POSITION_APP: _PortNum.ValueType  # 3
    """
    The built-in position messaging app.
    Payload is a Position message.
    ENCODING: Protobuf
    """
    NODEINFO_APP: _PortNum.ValueType  # 4
    """
    The built-in user info app.
    Payload is a User message.
    ENCODING: Protobuf
    """
    ROUTING_APP: _PortNum.ValueType  # 5
    """
    Protocol control packets for mesh protocol use.
    Payload is a Routing message.
    ENCODING: Protobuf
    """
    ADMIN_APP: _PortNum.ValueType  # 6
    """
    Admin control packets.
    Payload is a AdminMessage message.
    ENCODING: Protobuf
    """
    TEXT_MESSAGE_COMPRESSED_APP: _PortNum.ValueType  # 7
    """
    Compressed TEXT_MESSAGE payloads.
    ENCODING: UTF-8 Plaintext (?) with Unishox2 Compression
    NOTE: The Device Firmware converts a TEXT_MESSAGE_APP to TEXT_MESSAGE_COMPRESSED_APP if the compressed
    payload is shorter. There's no need for app developers to do this themselves. Also the firmware will decompress
    any incoming TEXT_MESSAGE_COMPRESSED_APP payload and convert to TEXT_MESSAGE_APP.
    """
    WAYPOINT_APP: _PortNum.ValueType  # 8
    """
    Waypoint payloads.
    Payload is a Waypoint message.
    ENCODING: Protobuf
    """
    AUDIO_APP: _PortNum.ValueType  # 9
    """
    Audio Payloads.
    Encapsulated codec2 packets. On 2.4 GHZ Bandwidths only for now
    ENCODING: codec2 audio frames
    NOTE: audio frames contain a 3 byte header (0xc0 0xde 0xc2) and a one byte marker for the decompressed bitrate.
    This marker comes from the 'moduleConfig.audio.bitrate' enum minus one.
    """
    DETECTION_SENSOR_APP: _PortNum.ValueType  # 10
    """
    Same as Text Message but originating from Detection Sensor Module.
    NOTE: This portnum traffic is not sent to the public MQTT starting at firmware version 2.2.9
    """
    ALERT_APP: _PortNum.ValueType  # 11
    """
    Same as Text Message but used for critical alerts.
    """
    REPLY_APP: _PortNum.ValueType  # 32
    """
    Provides a 'ping' service that replies to any packet it receives.
    Also serves as a small example module.
    ENCODING: ASCII Plaintext
    """
    IP_TUNNEL_APP: _PortNum.ValueType  # 33
    """
    Used for the python IP tunnel feature
    ENCODING: IP Packet. Handled by the python API, firmware ignores this one and pases on.
    """
    PAXCOUNTER_APP: _PortNum.ValueType  # 34
    """
    Paxcounter lib included in the firmware
    ENCODING: protobuf
    """
    SERIAL_APP: _PortNum.ValueType  # 64
    """
    Provides a hardware serial interface to send and receive from the Meshtastic network.
    Connect to the RX/TX pins of a device with 38400 8N1. Packets received from the Meshtastic
    network is forwarded to the RX pin while sending a packet to TX will go out to the Mesh network.
    Maximum packet size of 240 bytes.
    Module is disabled by default can be turned on by setting SERIAL_MODULE_ENABLED = 1 in SerialPlugh.cpp.
    ENCODING: binary undefined
    """
    STORE_FORWARD_APP: _PortNum.ValueType  # 65
    """
    STORE_FORWARD_APP (Work in Progress)
    Maintained by Jm Casler (MC Hamster) : jm@casler.org
    ENCODING: Protobuf
    """
    RANGE_TEST_APP: _PortNum.ValueType  # 66
    """
    Optional port for messages for the range test module.
    ENCODING: ASCII Plaintext
    NOTE: This portnum traffic is not sent to the public MQTT starting at firmware version 2.2.9
    """
    TELEMETRY_APP: _PortNum.ValueType  # 67
    """
    Provides a format to send and receive telemetry data from the Meshtastic network.
    Maintained by Charles Crossan (crossan007) : crossan007@gmail.com
    ENCODING: Protobuf
    """
    ZPS_APP: _PortNum.ValueType  # 68
    """
    Experimental tools for estimating node position without a GPS
    Maintained by Github user a-f-G-U-C (a Meshtastic contributor)
    Project files at https://github.com/a-f-G-U-C/Meshtastic-ZPS
    ENCODING: arrays of int64 fields
    """
    SIMULATOR_APP: _PortNum.ValueType  # 69
    """
    Used to let multiple instances of Linux native applications communicate
    as if they did using their LoRa chip.
    Maintained by GitHub user GUVWAF.
    Project files at https://github.com/GUVWAF/Meshtasticator
    ENCODING: Protobuf (?)
    """
    TRACEROUTE_APP: _PortNum.ValueType  # 70
    """
    Provides a traceroute functionality to show the route a packet towards
    a certain destination would take on the mesh. Contains a RouteDiscovery message as payload.
    ENCODING: Protobuf
    """
    NEIGHBORINFO_APP: _PortNum.ValueType  # 71
    """
    Aggregates edge info for the network by sending out a list of each node's neighbors
    ENCODING: Protobuf
    """
    ATAK_PLUGIN: _PortNum.ValueType  # 72
    """
    ATAK Plugin
    Portnum for payloads from the official Meshtastic ATAK plugin
    """
    MAP_REPORT_APP: _PortNum.ValueType  # 73
    """
    Provides unencrypted information about a node for consumption by a map via MQTT
    """
    POWERSTRESS_APP: _PortNum.ValueType  # 74
    """
    PowerStress based monitoring support (for automated power consumption testing)
    """
    PRIVATE_APP: _PortNum.ValueType  # 256
    """
    Private applications should use portnums >= 256.
    To simplify initial development and testing you can use "PRIVATE_APP"
    in your code without needing to rebuild protobuf files (via [regen-protos.sh](https://github.com/meshtastic/firmware/blob/master/bin/regen-protos.sh))
    """
    ATAK_FORWARDER: _PortNum.ValueType  # 257
    """
    ATAK Forwarder Module https://github.com/paulmandal/atak-forwarder
    ENCODING: libcotshrink
    """
    MAX: _PortNum.ValueType  # 511
    """
    Currently we limit port nums to no higher than this value
    """

class PortNum(_PortNum, metaclass=_PortNumEnumTypeWrapper):
    """
    For any new 'apps' that run on the device or via sister apps on phones/PCs they should pick and use a
    unique 'portnum' for their application.
    If you are making a new app using meshtastic, please send in a pull request to add your 'portnum' to this
    master table.
    PortNums should be assigned in the following range:
    0-63   Core Meshtastic use, do not use for third party apps
    64-127 Registered 3rd party apps, send in a pull request that adds a new entry to portnums.proto to  register your application
    256-511 Use one of these portnums for your private applications that you don't want to register publically
    All other values are reserved.
    Note: This was formerly a Type enum named 'typ' with the same id #
    We have change to this 'portnum' based scheme for specifying app handlers for particular payloads.
    This change is backwards compatible by treating the legacy OPAQUE/CLEAR_TEXT values identically.
    """

UNKNOWN_APP: PortNum.ValueType  # 0
"""
Deprecated: do not use in new code (formerly called OPAQUE)
A message sent from a device outside of the mesh, in a form the mesh does not understand
NOTE: This must be 0, because it is documented in IMeshService.aidl to be so
ENCODING: binary undefined
"""
TEXT_MESSAGE_APP: PortNum.ValueType  # 1
"""
A simple UTF-8 text message, which even the little micros in the mesh
can understand and show on their screen eventually in some circumstances
even signal might send messages in this form (see below)
ENCODING: UTF-8 Plaintext (?)
"""
REMOTE_HARDWARE_APP: PortNum.ValueType  # 2
"""
Reserved for built-in GPIO/example app.
See remote_hardware.proto/HardwareMessage for details on the message sent/received to this port number
ENCODING: Protobuf
"""
POSITION_APP: PortNum.ValueType  # 3
"""
The built-in position messaging app.
Payload is a Position message.
ENCODING: Protobuf
"""
NODEINFO_APP: PortNum.ValueType  # 4
"""
The built-in user info app.
Payload is a User message.
ENCODING: Protobuf
"""
ROUTING_APP: PortNum.ValueType  # 5
"""
Protocol control packets for mesh protocol use.
Payload is a Routing message.
ENCODING: Protobuf
"""
ADMIN_APP: PortNum.ValueType  # 6
"""
Admin control packets.
Payload is a AdminMessage message.
ENCODING: Protobuf
"""
TEXT_MESSAGE_COMPRESSED_APP: PortNum.ValueType  # 7
"""
Compressed TEXT_MESSAGE payloads.
ENCODING: UTF-8 Plaintext (?) with Unishox2 Compression
NOTE: The Device Firmware converts a TEXT_MESSAGE_APP to TEXT_MESSAGE_COMPRESSED_APP if the compressed
payload is shorter. There's no need for app developers to do this themselves. Also the firmware will decompress
any incoming TEXT_MESSAGE_COMPRESSED_APP payload and convert to TEXT_MESSAGE_APP.
"""
WAYPOINT_APP: PortNum.ValueType  # 8
"""
Waypoint payloads.
Payload is a Waypoint message.
ENCODING: Protobuf
"""
AUDIO_APP: PortNum.ValueType  # 9
"""
Audio Payloads.
Encapsulated codec2 packets. On 2.4 GHZ Bandwidths only for now
ENCODING: codec2 audio frames
NOTE: audio frames contain a 3 byte header (0xc0 0xde 0xc2) and a one byte marker for the decompressed bitrate.
This marker comes from the 'moduleConfig.audio.bitrate' enum minus one.
"""
DETECTION_SENSOR_APP: PortNum.ValueType  # 10
"""
Same as Text Message but originating from Detection Sensor Module.
NOTE: This portnum traffic is not sent to the public MQTT starting at firmware version 2.2.9
"""
ALERT_APP: PortNum.ValueType  # 11
"""
Same as Text Message but used for critical alerts.
"""
REPLY_APP: PortNum.ValueType  # 32
"""
Provides a 'ping' service that replies to any packet it receives.
Also serves as a small example module.
ENCODING: ASCII Plaintext
"""
IP_TUNNEL_APP: PortNum.ValueType  # 33
"""
Used for the python IP tunnel feature
ENCODING: IP Packet. Handled by the python API, firmware ignores this one and pases on.
"""
PAXCOUNTER_APP: PortNum.ValueType  # 34
"""
Paxcounter lib included in the firmware
ENCODING: protobuf
"""
SERIAL_APP: PortNum.ValueType  # 64
"""
Provides a hardware serial interface to send and receive from the Meshtastic network.
Connect to the RX/TX pins of a device with 38400 8N1. Packets received from the Meshtastic
network is forwarded to the RX pin while sending a packet to TX will go out to the Mesh network.
Maximum packet size of 240 bytes.
Module is disabled by default can be turned on by setting SERIAL_MODULE_ENABLED = 1 in SerialPlugh.cpp.
ENCODING: binary undefined
"""
STORE_FORWARD_APP: PortNum.ValueType  # 65
"""
STORE_FORWARD_APP (Work in Progress)
Maintained by Jm Casler (MC Hamster) : jm@casler.org
ENCODING: Protobuf
"""
RANGE_TEST_APP: PortNum.ValueType  # 66
"""
Optional port for messages for the range test module.
ENCODING: ASCII Plaintext
NOTE: This portnum traffic is not sent to the public MQTT starting at firmware version 2.2.9
"""
TELEMETRY_APP: PortNum.ValueType  # 67
"""
Provides a format to send and receive telemetry data from the Meshtastic network.
Maintained by Charles Crossan (crossan007) : crossan007@gmail.com
ENCODING: Protobuf
"""
ZPS_APP: PortNum.ValueType  # 68
"""
Experimental tools for estimating node position without a GPS
Maintained by Github user a-f-G-U-C (a Meshtastic contributor)
Project files at https://github.com/a-f-G-U-C/Meshtastic-ZPS
ENCODING: arrays of int64 fields
"""
SIMULATOR_APP: PortNum.ValueType  # 69
"""
Used to let multiple instances of Linux native applications communicate
as if they did using their LoRa chip.
Maintained by GitHub user GUVWAF.
Project files at https://github.com/GUVWAF/Meshtasticator
ENCODING: Protobuf (?)
"""
TRACEROUTE_APP: PortNum.ValueType  # 70
"""
Provides a traceroute functionality to show the route a packet towards
a certain destination would take on the mesh. Contains a RouteDiscovery message as payload.
ENCODING: Protobuf
"""
NEIGHBORINFO_APP: PortNum.ValueType  # 71
"""
Aggregates edge info for the network by sending out a list of each node's neighbors
ENCODING: Protobuf
"""
ATAK_PLUGIN: PortNum.ValueType  # 72
"""
ATAK Plugin
Portnum for payloads from the official Meshtastic ATAK plugin
"""
MAP_REPORT_APP: PortNum.ValueType  # 73
"""
Provides unencrypted information about a node for consumption by a map via MQTT
"""
POWERSTRESS_APP: PortNum.ValueType  # 74
"""
PowerStress based monitoring support (for automated power consumption testing)
"""
PRIVATE_APP: PortNum.ValueType  # 256
"""
Private applications should use portnums >= 256.
To simplify initial development and testing you can use "PRIVATE_APP"
in your code without needing to rebuild protobuf files (via [regen-protos.sh](https://github.com/meshtastic/firmware/blob/master/bin/regen-protos.sh))
"""
ATAK_FORWARDER: PortNum.ValueType  # 257
"""
ATAK Forwarder Module https://github.com/paulmandal/atak-forwarder
ENCODING: libcotshrink
"""
MAX: PortNum.ValueType  # 511
"""
Currently we limit port nums to no higher than this value
"""
global___PortNum = PortNum



"""Program to scan for hardware
   To run: python examples/scan_for_devices.py
"""

import sys

from meshtastic.util import (
    active_ports_on_supported_devices,
    detect_supported_devices,
    get_unique_vendor_ids,
)

# simple arg check
if len(sys.argv) != 1:
    print(f"usage: {sys.argv[0]}")
    print("Detect which device we might have.")
    sys.exit(3)

vids = get_unique_vendor_ids()
print(f"Searching for all devices with these vendor ids {vids}")

sds = detect_supported_devices()
if len(sds) > 0:
    print("Detected possible devices:")
for d in sds:
    print(f" name:{d.name}{d.version} firmware:{d.for_firmware}")

ports = active_ports_on_supported_devices(sds)
print(f"ports:{ports}")



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _Team:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _TeamEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Team.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    Unspecifed_Color: _Team.ValueType  # 0
    """
    Unspecifed
    """
    White: _Team.ValueType  # 1
    """
    White
    """
    Yellow: _Team.ValueType  # 2
    """
    Yellow
    """
    Orange: _Team.ValueType  # 3
    """
    Orange
    """
    Magenta: _Team.ValueType  # 4
    """
    Magenta
    """
    Red: _Team.ValueType  # 5
    """
    Red
    """
    Maroon: _Team.ValueType  # 6
    """
    Maroon
    """
    Purple: _Team.ValueType  # 7
    """
    Purple
    """
    Dark_Blue: _Team.ValueType  # 8
    """
    Dark Blue
    """
    Blue: _Team.ValueType  # 9
    """
    Blue
    """
    Cyan: _Team.ValueType  # 10
    """
    Cyan
    """
    Teal: _Team.ValueType  # 11
    """
    Teal
    """
    Green: _Team.ValueType  # 12
    """
    Green
    """
    Dark_Green: _Team.ValueType  # 13
    """
    Dark Green
    """
    Brown: _Team.ValueType  # 14
    """
    Brown
    """

class Team(_Team, metaclass=_TeamEnumTypeWrapper): ...

Unspecifed_Color: Team.ValueType  # 0
"""
Unspecifed
"""
White: Team.ValueType  # 1
"""
White
"""
Yellow: Team.ValueType  # 2
"""
Yellow
"""
Orange: Team.ValueType  # 3
"""
Orange
"""
Magenta: Team.ValueType  # 4
"""
Magenta
"""
Red: Team.ValueType  # 5
"""
Red
"""
Maroon: Team.ValueType  # 6
"""
Maroon
"""
Purple: Team.ValueType  # 7
"""
Purple
"""
Dark_Blue: Team.ValueType  # 8
"""
Dark Blue
"""
Blue: Team.ValueType  # 9
"""
Blue
"""
Cyan: Team.ValueType  # 10
"""
Cyan
"""
Teal: Team.ValueType  # 11
"""
Teal
"""
Green: Team.ValueType  # 12
"""
Green
"""
Dark_Green: Team.ValueType  # 13
"""
Dark Green
"""
Brown: Team.ValueType  # 14
"""
Brown
"""
global___Team = Team

class _MemberRole:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _MemberRoleEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_MemberRole.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    Unspecifed: _MemberRole.ValueType  # 0
    """
    Unspecifed
    """
    TeamMember: _MemberRole.ValueType  # 1
    """
    Team Member
    """
    TeamLead: _MemberRole.ValueType  # 2
    """
    Team Lead
    """
    HQ: _MemberRole.ValueType  # 3
    """
    Headquarters
    """
    Sniper: _MemberRole.ValueType  # 4
    """
    Airsoft enthusiast
    """
    Medic: _MemberRole.ValueType  # 5
    """
    Medic
    """
    ForwardObserver: _MemberRole.ValueType  # 6
    """
    ForwardObserver
    """
    RTO: _MemberRole.ValueType  # 7
    """
    Radio Telephone Operator
    """
    K9: _MemberRole.ValueType  # 8
    """
    Doggo
    """

class MemberRole(_MemberRole, metaclass=_MemberRoleEnumTypeWrapper):
    """
    Role of the group member
    """

Unspecifed: MemberRole.ValueType  # 0
"""
Unspecifed
"""
TeamMember: MemberRole.ValueType  # 1
"""
Team Member
"""
TeamLead: MemberRole.ValueType  # 2
"""
Team Lead
"""
HQ: MemberRole.ValueType  # 3
"""
Headquarters
"""
Sniper: MemberRole.ValueType  # 4
"""
Airsoft enthusiast
"""
Medic: MemberRole.ValueType  # 5
"""
Medic
"""
ForwardObserver: MemberRole.ValueType  # 6
"""
ForwardObserver
"""
RTO: MemberRole.ValueType  # 7
"""
Radio Telephone Operator
"""
K9: MemberRole.ValueType  # 8
"""
Doggo
"""
global___MemberRole = MemberRole

@typing.final
class TAKPacket(google.protobuf.message.Message):
    """
    Packets for the official ATAK Plugin
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IS_COMPRESSED_FIELD_NUMBER: builtins.int
    CONTACT_FIELD_NUMBER: builtins.int
    GROUP_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    PLI_FIELD_NUMBER: builtins.int
    CHAT_FIELD_NUMBER: builtins.int
    DETAIL_FIELD_NUMBER: builtins.int
    is_compressed: builtins.bool
    """
    Are the payloads strings compressed for LoRA transport?
    """
    detail: builtins.bytes
    """
    Generic CoT detail XML
    May be compressed / truncated by the sender (EUD)
    """
    @property
    def contact(self) -> global___Contact:
        """
        The contact / callsign for ATAK user
        """

    @property
    def group(self) -> global___Group:
        """
        The group for ATAK user
        """

    @property
    def status(self) -> global___Status:
        """
        The status of the ATAK EUD
        """

    @property
    def pli(self) -> global___PLI:
        """
        TAK position report
        """

    @property
    def chat(self) -> global___GeoChat:
        """
        ATAK GeoChat message
        """

    def __init__(
        self,
        *,
        is_compressed: builtins.bool = ...,
        contact: global___Contact | None = ...,
        group: global___Group | None = ...,
        status: global___Status | None = ...,
        pli: global___PLI | None = ...,
        chat: global___GeoChat | None = ...,
        detail: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["chat", b"chat", "contact", b"contact", "detail", b"detail", "group", b"group", "payload_variant", b"payload_variant", "pli", b"pli", "status", b"status"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["chat", b"chat", "contact", b"contact", "detail", b"detail", "group", b"group", "is_compressed", b"is_compressed", "payload_variant", b"payload_variant", "pli", b"pli", "status", b"status"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["payload_variant", b"payload_variant"]) -> typing.Literal["pli", "chat", "detail"] | None: ...

global___TAKPacket = TAKPacket

@typing.final
class GeoChat(google.protobuf.message.Message):
    """
    ATAK GeoChat message
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MESSAGE_FIELD_NUMBER: builtins.int
    TO_FIELD_NUMBER: builtins.int
    TO_CALLSIGN_FIELD_NUMBER: builtins.int
    message: builtins.str
    """
    The text message
    """
    to: builtins.str
    """
    Uid recipient of the message
    """
    to_callsign: builtins.str
    """
    Callsign of the recipient for the message
    """
    def __init__(
        self,
        *,
        message: builtins.str = ...,
        to: builtins.str | None = ...,
        to_callsign: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_to", b"_to", "_to_callsign", b"_to_callsign", "to", b"to", "to_callsign", b"to_callsign"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_to", b"_to", "_to_callsign", b"_to_callsign", "message", b"message", "to", b"to", "to_callsign", b"to_callsign"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_to", b"_to"]) -> typing.Literal["to"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_to_callsign", b"_to_callsign"]) -> typing.Literal["to_callsign"] | None: ...

global___GeoChat = GeoChat

@typing.final
class Group(google.protobuf.message.Message):
    """
    ATAK Group
    <__group role='Team Member' name='Cyan'/>
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROLE_FIELD_NUMBER: builtins.int
    TEAM_FIELD_NUMBER: builtins.int
    role: global___MemberRole.ValueType
    """
    Role of the group member
    """
    team: global___Team.ValueType
    """
    Team (color)
    Default Cyan
    """
    def __init__(
        self,
        *,
        role: global___MemberRole.ValueType = ...,
        team: global___Team.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["role", b"role", "team", b"team"]) -> None: ...

global___Group = Group

@typing.final
class Status(google.protobuf.message.Message):
    """
    ATAK EUD Status
    <status battery='100' />
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BATTERY_FIELD_NUMBER: builtins.int
    battery: builtins.int
    """
    Battery level
    """
    def __init__(
        self,
        *,
        battery: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["battery", b"battery"]) -> None: ...

global___Status = Status

@typing.final
class Contact(google.protobuf.message.Message):
    """
    ATAK Contact
    <contact endpoint='0.0.0.0:4242:tcp' phone='+12345678' callsign='FALKE'/>
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CALLSIGN_FIELD_NUMBER: builtins.int
    DEVICE_CALLSIGN_FIELD_NUMBER: builtins.int
    callsign: builtins.str
    """
    Callsign
    """
    device_callsign: builtins.str
    """
    Device callsign

    IP address of endpoint in integer form (0.0.0.0 default)
    """
    def __init__(
        self,
        *,
        callsign: builtins.str = ...,
        device_callsign: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["callsign", b"callsign", "device_callsign", b"device_callsign"]) -> None: ...

global___Contact = Contact

@typing.final
class PLI(google.protobuf.message.Message):
    """
    Position Location Information from ATAK
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LATITUDE_I_FIELD_NUMBER: builtins.int
    LONGITUDE_I_FIELD_NUMBER: builtins.int
    ALTITUDE_FIELD_NUMBER: builtins.int
    SPEED_FIELD_NUMBER: builtins.int
    COURSE_FIELD_NUMBER: builtins.int
    latitude_i: builtins.int
    """
    The new preferred location encoding, multiply by 1e-7 to get degrees
    in floating point
    """
    longitude_i: builtins.int
    """
    The new preferred location encoding, multiply by 1e-7 to get degrees
    in floating point
    """
    altitude: builtins.int
    """
    Altitude (ATAK prefers HAE)
    """
    speed: builtins.int
    """
    Speed
    """
    course: builtins.int
    """
    Course in degrees
    """
    def __init__(
        self,
        *,
        latitude_i: builtins.int = ...,
        longitude_i: builtins.int = ...,
        altitude: builtins.int = ...,
        speed: builtins.int = ...,
        course: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["altitude", b"altitude", "course", b"course", "latitude_i", b"latitude_i", "longitude_i", b"longitude_i", "speed", b"speed"]) -> None: ...

global___PLI = PLI



"""Simple program to demo how to use meshtastic library.
   To run: python examples/set_owner.py Bobby 333
"""

import sys

import meshtastic
import meshtastic.serial_interface

# simple arg check
if len(sys.argv) < 2:
    print(f"usage: {sys.argv[0]} long_name [short_name]")
    sys.exit(3)

iface = meshtastic.serial_interface.SerialInterface()
long_name = sys.argv[1]
short_name = None
if len(sys.argv) > 2:
    short_name = sys.argv[2]
iface.localNode.setOwner(long_name, short_name)
iface.close()



"""Meshtastic unit tests for mesh_interface.py"""

import logging
import re
from unittest.mock import MagicMock, patch

import pytest
from hypothesis import given, strategies as st

from ..protobuf import mesh_pb2, config_pb2
from .. import BROADCAST_ADDR, LOCAL_ADDR
from ..mesh_interface import MeshInterface, _timeago
from ..node import Node
from ..slog import LogSet
from ..powermon import SimPowerSupply

# TODO
# from ..config import Config
from ..util import Timeout


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_MeshInterface(capsys):
    """Test that we can instantiate a MeshInterface"""
    iface = MeshInterface(noProto=True)

    NODE_ID = "!9388f81c"
    NODE_NUM = 2475227164
    node = {
            "num": NODE_NUM,
            "user": {
                "id": NODE_ID,
                "longName": "Unknown f81c",
                "shortName": "?1C",
                "macaddr": "RBeTiPgc",
                "hwModel": "TBEAM",
            },
            "position": {},
            "lastHeard": 1640204888,
        }


    iface.nodes = {NODE_ID: node}
    iface.nodesByNum = {NODE_NUM: node}

    myInfo = MagicMock()
    iface.myInfo = myInfo

    iface.localNode.localConfig.lora.CopyFrom(config_pb2.Config.LoRaConfig())

    # Also get some coverage of the structured logging/power meter stuff by turning it on as well
    log_set = LogSet(iface, None, SimPowerSupply())

    iface.showInfo()
    iface.localNode.showInfo()
    iface.showNodes()
    iface.sendText("hello")
    iface.close()
    log_set.close()
    out, err = capsys.readouterr()
    assert re.search(r"Owner: None \(None\)", out, re.MULTILINE)
    assert re.search(r"Nodes", out, re.MULTILINE)
    assert re.search(r"Preferences", out, re.MULTILINE)
    assert re.search(r"Channels", out, re.MULTILINE)
    assert re.search(r"Primary channel URL", out, re.MULTILINE)
    assert err == ""


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_getMyUser(iface_with_nodes):
    """Test getMyUser()"""
    iface = iface_with_nodes
    iface.myInfo.my_node_num = 2475227164
    myuser = iface.getMyUser()
    assert myuser is not None
    assert myuser["id"] == "!9388f81c"


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_getLongName(iface_with_nodes):
    """Test getLongName()"""
    iface = iface_with_nodes
    iface.myInfo.my_node_num = 2475227164
    mylongname = iface.getLongName()
    assert mylongname == "Unknown f81c"


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_getShortName(iface_with_nodes):
    """Test getShortName()."""
    iface = iface_with_nodes
    iface.myInfo.my_node_num = 2475227164
    myshortname = iface.getShortName()
    assert myshortname == "?1C"


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_handlePacketFromRadio_no_from(capsys):
    """Test _handlePacketFromRadio with no 'from' in the mesh packet."""
    iface = MeshInterface(noProto=True)
    meshPacket = mesh_pb2.MeshPacket()
    iface._handlePacketFromRadio(meshPacket)
    out, err = capsys.readouterr()
    assert re.search(r"Device returned a packet we sent, ignoring", out, re.MULTILINE)
    assert err == ""


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_handlePacketFromRadio_with_a_portnum(caplog):
    """Test _handlePacketFromRadio with a portnum
    Since we have an attribute called 'from', we cannot simply 'set' it.
    Had to implement a hack just to be able to test some code.
    """
    iface = MeshInterface(noProto=True)
    meshPacket = mesh_pb2.MeshPacket()
    meshPacket.decoded.payload = b""
    meshPacket.decoded.portnum = 1
    with caplog.at_level(logging.WARNING):
        iface._handlePacketFromRadio(meshPacket, hack=True)
    assert re.search(r"Not populating fromId", caplog.text, re.MULTILINE)


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_handlePacketFromRadio_no_portnum(caplog):
    """Test _handlePacketFromRadio without a portnum"""
    iface = MeshInterface(noProto=True)
    meshPacket = mesh_pb2.MeshPacket()
    meshPacket.decoded.payload = b""
    with caplog.at_level(logging.WARNING):
        iface._handlePacketFromRadio(meshPacket, hack=True)
    assert re.search(r"Not populating fromId", caplog.text, re.MULTILINE)


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_getNode_with_local():
    """Test getNode"""
    iface = MeshInterface(noProto=True)
    anode = iface.getNode(LOCAL_ADDR)
    assert anode == iface.localNode


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_getNode_not_local(caplog):
    """Test getNode not local"""
    iface = MeshInterface(noProto=True)
    anode = MagicMock(autospec=Node)
    with caplog.at_level(logging.DEBUG):
        with patch("meshtastic.node.Node", return_value=anode):
            another_node = iface.getNode("bar2")
            assert another_node != iface.localNode
    assert re.search(r"About to requestChannels", caplog.text, re.MULTILINE)


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_getNode_not_local_timeout(capsys):
    """Test getNode not local, simulate timeout"""
    iface = MeshInterface(noProto=True)
    anode = MagicMock(autospec=Node)
    anode.waitForConfig.return_value = False
    with patch("meshtastic.node.Node", return_value=anode):
        with pytest.raises(SystemExit) as pytest_wrapped_e:
            iface.getNode("bar2")
        assert pytest_wrapped_e.type == SystemExit
        assert pytest_wrapped_e.value.code == 1
        out, err = capsys.readouterr()
        assert re.match(r"Timed out trying to retrieve channel info, retrying", out)
        assert err == ""

@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_getNode_not_local_timeout_attempts(capsys):
    """Test getNode not local, simulate timeout"""
    iface = MeshInterface(noProto=True)
    anode = MagicMock(autospec=Node)
    anode.waitForConfig.return_value = False
    with patch("meshtastic.node.Node", return_value=anode):
        with pytest.raises(SystemExit) as pytest_wrapped_e:
            iface.getNode("bar2", requestChannelAttempts=2)
        assert pytest_wrapped_e.type == SystemExit
        assert pytest_wrapped_e.value.code == 1
        out, err = capsys.readouterr()
        assert out == 'Timed out trying to retrieve channel info, retrying\nError: Timed out waiting for channels, giving up\n'
        assert err == ""


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_sendPosition(caplog):
    """Test sendPosition"""
    iface = MeshInterface(noProto=True)
    with caplog.at_level(logging.DEBUG):
        iface.sendPosition()
    iface.close()
    # assert re.search(r"p.time:", caplog.text, re.MULTILINE)


# TODO
# @pytest.mark.unit
# @pytest.mark.usefixtures("reset_mt_config")
# def test_close_with_heartbeatTimer(caplog):
#    """Test close() with heartbeatTimer"""
#    iface = MeshInterface(noProto=True)
#    anode = Node('foo', 'bar')
#    aconfig = Config()
#    aonfig.preferences.phone_timeout_secs = 10
#    anode.config = aconfig
#    iface.localNode = anode
#    assert iface.heartbeatTimer is None
#    with caplog.at_level(logging.DEBUG):
#        iface._startHeartbeat()
#        assert iface.heartbeatTimer is not None
#        iface.close()


# TODO
# @pytest.mark.unit
# @pytest.mark.usefixtures("reset_mt_config")
# def test_handleFromRadio_empty_payload(caplog):
#    """Test _handleFromRadio"""
#    iface = MeshInterface(noProto=True)
#    with caplog.at_level(logging.DEBUG):
#        iface._handleFromRadio(b'')
#    iface.close()
#    assert re.search(r'Unexpected FromRadio payload', caplog.text, re.MULTILINE)


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_handleFromRadio_with_my_info(caplog):
    """Test _handleFromRadio with my_info"""
    # Note: I captured the '--debug --info' for the bytes below.
    # It "translates" to this:
    # my_info {
    #  my_node_num: 682584012
    #  firmware_version: "1.2.49.5354c49"
    #  reboot_count: 13
    #  bitrate: 17.088470458984375
    #  message_timeout_msec: 300000
    #  min_app_version: 20200
    #  max_channels: 8
    #  has_wifi: true
    # }
    from_radio_bytes = b"\x1a,\x08\xcc\xcf\xbd\xc5\x02\x18\r2\x0e1.2.49.5354c49P\r]0\xb5\x88Ah\xe0\xa7\x12p\xe8\x9d\x01x\x08\x90\x01\x01"
    iface = MeshInterface(noProto=True)
    with caplog.at_level(logging.DEBUG):
        iface._handleFromRadio(from_radio_bytes)
    iface.close()
    assert re.search(r"Received from radio: my_info {", caplog.text, re.MULTILINE)
    assert re.search(r"my_node_num: 682584012", caplog.text, re.MULTILINE)


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_handleFromRadio_with_node_info(caplog, capsys):
    """Test _handleFromRadio with node_info"""
    # Note: I captured the '--debug --info' for the bytes below.
    # It "translates" to this:
    # node_info {
    #  num: 682584012
    #  user {
    #    id: "!28af67cc"
    #    long_name: "Unknown 67cc"
    #    short_name: "?CC"
    #    macaddr: "$o(\257g\314"
    #    hw_model: HELTEC_V2_1
    #  }
    #  position {
    #    }
    #  }

    from_radio_bytes = b'"2\x08\xcc\xcf\xbd\xc5\x02\x12(\n\t!28af67cc\x12\x0cUnknown 67cc\x1a\x03?CC"\x06$o(\xafg\xcc0\n\x1a\x00'
    iface = MeshInterface(noProto=True)
    with caplog.at_level(logging.DEBUG):
        iface._startConfig()
        iface._handleFromRadio(from_radio_bytes)
        assert re.search(r"Received from radio: node_info {", caplog.text, re.MULTILINE)
        assert re.search(r"682584012", caplog.text, re.MULTILINE)
        # validate some of showNodes() output
        iface.showNodes()
        out, err = capsys.readouterr()
        assert re.search(r" 1 ", out, re.MULTILINE)
        assert re.search(r"│ Unknown 67cc │ ", out, re.MULTILINE)
        assert re.search(r"│\s+!28af67cc\s+│\s+67cc\s+|", out, re.MULTILINE)
        assert err == ""
        iface.close()


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_handleFromRadio_with_node_info_tbeam1(caplog, capsys):
    """Test _handleFromRadio with node_info"""
    # Note: Captured the '--debug --info' for the bytes below.
    # pylint: disable=C0301
    from_radio_bytes = b'"=\x08\x80\xf8\xc8\xf6\x07\x12"\n\t!7ed23c00\x12\x07TBeam 1\x1a\x02T1"\x06\x94\xb9~\xd2<\x000\x04\x1a\x07 ]MN\x01\xbea%\xad\x01\xbea=\x00\x00,A'
    iface = MeshInterface(noProto=True)
    with caplog.at_level(logging.DEBUG):
        iface._startConfig()
        iface._handleFromRadio(from_radio_bytes)
        assert re.search(r"Received nodeinfo", caplog.text, re.MULTILINE)
        assert re.search(r"TBeam 1", caplog.text, re.MULTILINE)
        assert re.search(r"2127707136", caplog.text, re.MULTILINE)
        # validate some of showNodes() output
        iface.showNodes()
        out, err = capsys.readouterr()
        assert re.search(r" 1 ", out, re.MULTILINE)
        assert re.search(r"│ TBeam 1 │ ", out, re.MULTILINE)
        assert re.search(r"│ !7ed23c00 │", out, re.MULTILINE)
        assert err == ""
        iface.close()


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_handleFromRadio_with_node_info_tbeam_with_bad_data(caplog):
    """Test _handleFromRadio with node_info with some bad data (issue#172) - ensure we do not throw exception"""
    # Note: Captured the '--debug --info' for the bytes below.
    from_radio_bytes = b'"\x17\x08\xdc\x8a\x8a\xae\x02\x12\x08"\x06\x00\x00\x00\x00\x00\x00\x1a\x00=\x00\x00\xb8@'
    iface = MeshInterface(noProto=True)
    with caplog.at_level(logging.DEBUG):
        iface._startConfig()
        iface._handleFromRadio(from_radio_bytes)


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_MeshInterface_sendToRadioImpl(caplog):
    """Test _sendToRadioImp()"""
    iface = MeshInterface(noProto=True)
    with caplog.at_level(logging.DEBUG):
        iface._sendToRadioImpl("foo")
    assert re.search(r"Subclass must provide toradio", caplog.text, re.MULTILINE)
    iface.close()


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_MeshInterface_sendToRadio_no_proto(caplog):
    """Test sendToRadio()"""
    iface = MeshInterface()
    with caplog.at_level(logging.DEBUG):
        iface._sendToRadioImpl("foo")
    assert re.search(r"Subclass must provide toradio", caplog.text, re.MULTILINE)
    iface.close()


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_sendData_too_long(caplog):
    """Test when data payload is too big"""
    iface = MeshInterface(noProto=True)
    some_large_text = b"This is a long text that will be too long for send text."
    some_large_text += b"This is a long text that will be too long for send text."
    some_large_text += b"This is a long text that will be too long for send text."
    some_large_text += b"This is a long text that will be too long for send text."
    some_large_text += b"This is a long text that will be too long for send text."
    some_large_text += b"This is a long text that will be too long for send text."
    some_large_text += b"This is a long text that will be too long for send text."
    some_large_text += b"This is a long text that will be too long for send text."
    some_large_text += b"This is a long text that will be too long for send text."
    some_large_text += b"This is a long text that will be too long for send text."
    some_large_text += b"This is a long text that will be too long for send text."
    some_large_text += b"This is a long text that will be too long for send text."
    with caplog.at_level(logging.DEBUG):
        with pytest.raises(MeshInterface.MeshInterfaceError) as pytest_wrapped_e:
            iface.sendData(some_large_text)
            assert re.search("Data payload too big", caplog.text, re.MULTILINE)
        assert pytest_wrapped_e.type == MeshInterface.MeshInterfaceError
    iface.close()


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_sendData_unknown_app(capsys):
    """Test sendData when unknown app"""
    iface = MeshInterface(noProto=True)
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        iface.sendData(b"hello", portNum=0)
    out, err = capsys.readouterr()
    assert re.search(r"Warning: A non-zero port number", out, re.MULTILINE)
    assert err == ""
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 1


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_sendPosition_with_a_position(caplog):
    """Test sendPosition when lat/long/alt"""
    iface = MeshInterface(noProto=True)
    with caplog.at_level(logging.DEBUG):
        iface.sendPosition(latitude=40.8, longitude=-111.86, altitude=201)
        assert re.search(r"p.latitude_i:408", caplog.text, re.MULTILINE)
        assert re.search(r"p.longitude_i:-11186", caplog.text, re.MULTILINE)
        assert re.search(r"p.altitude:201", caplog.text, re.MULTILINE)


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_sendPacket_with_no_destination(capsys):
    """Test _sendPacket()"""
    iface = MeshInterface(noProto=True)
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        iface._sendPacket(b"", destinationId=None)
    out, err = capsys.readouterr()
    assert re.search(r"Warning: destinationId must not be None", out, re.MULTILINE)
    assert err == ""
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 1


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_sendPacket_with_destination_as_int(caplog):
    """Test _sendPacket() with int as a destination"""
    iface = MeshInterface(noProto=True)
    with caplog.at_level(logging.DEBUG):
        meshPacket = mesh_pb2.MeshPacket()
        iface._sendPacket(meshPacket, destinationId=123)
        assert re.search(r"Not sending packet", caplog.text, re.MULTILINE)


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_sendPacket_with_destination_starting_with_a_bang(caplog):
    """Test _sendPacket() with int as a destination"""
    iface = MeshInterface(noProto=True)
    with caplog.at_level(logging.DEBUG):
        meshPacket = mesh_pb2.MeshPacket()
        iface._sendPacket(meshPacket, destinationId="!1234")
        assert re.search(r"Not sending packet", caplog.text, re.MULTILINE)


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_sendPacket_with_destination_as_BROADCAST_ADDR(caplog):
    """Test _sendPacket() with BROADCAST_ADDR as a destination"""
    iface = MeshInterface(noProto=True)
    with caplog.at_level(logging.DEBUG):
        meshPacket = mesh_pb2.MeshPacket()
        iface._sendPacket(meshPacket, destinationId=BROADCAST_ADDR)
        assert re.search(r"Not sending packet", caplog.text, re.MULTILINE)


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_sendPacket_with_destination_as_LOCAL_ADDR_no_myInfo(capsys):
    """Test _sendPacket() with LOCAL_ADDR as a destination with no myInfo"""
    iface = MeshInterface(noProto=True)
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        meshPacket = mesh_pb2.MeshPacket()
        iface._sendPacket(meshPacket, destinationId=LOCAL_ADDR)
    out, err = capsys.readouterr()
    assert re.search(r"Warning: No myInfo", out, re.MULTILINE)
    assert err == ""
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 1


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_sendPacket_with_destination_as_LOCAL_ADDR_with_myInfo(caplog):
    """Test _sendPacket() with LOCAL_ADDR as a destination with myInfo"""
    iface = MeshInterface(noProto=True)
    myInfo = MagicMock()
    iface.myInfo = myInfo
    iface.myInfo.my_node_num = 1
    with caplog.at_level(logging.DEBUG):
        meshPacket = mesh_pb2.MeshPacket()
        iface._sendPacket(meshPacket, destinationId=LOCAL_ADDR)
        assert re.search(r"Not sending packet", caplog.text, re.MULTILINE)


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_sendPacket_with_destination_is_blank_with_nodes(capsys, iface_with_nodes):
    """Test _sendPacket() with '' as a destination with myInfo"""
    iface = iface_with_nodes
    meshPacket = mesh_pb2.MeshPacket()
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        iface._sendPacket(meshPacket, destinationId="")
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 1
    out, err = capsys.readouterr()
    assert re.match(r"Warning: NodeId  not found in DB", out, re.MULTILINE)
    assert err == ""


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_sendPacket_with_destination_is_blank_without_nodes(caplog, iface_with_nodes):
    """Test _sendPacket() with '' as a destination with myInfo"""
    iface = iface_with_nodes
    iface.nodes = None
    meshPacket = mesh_pb2.MeshPacket()
    with caplog.at_level(logging.WARNING):
        iface._sendPacket(meshPacket, destinationId="")
    assert re.search(r"Warning: There were no self.nodes.", caplog.text, re.MULTILINE)


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_getMyNodeInfo():
    """Test getMyNodeInfo()"""
    iface = MeshInterface(noProto=True)
    anode = iface.getNode(LOCAL_ADDR)
    iface.nodesByNum = {1: anode}
    assert iface.nodesByNum.get(1) == anode
    myInfo = MagicMock()
    iface.myInfo = myInfo
    iface.myInfo.my_node_num = 1
    myinfo = iface.getMyNodeInfo()
    assert myinfo == anode


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_generatePacketId(capsys):
    """Test _generatePacketId() when no currentPacketId (not connected)"""
    iface = MeshInterface(noProto=True)
    # not sure when this condition would ever happen... but we can simulate it
    iface.currentPacketId = None
    assert iface.currentPacketId is None
    with pytest.raises(MeshInterface.MeshInterfaceError) as pytest_wrapped_e:
        iface._generatePacketId()
        out, err = capsys.readouterr()
        assert re.search(
            r"Not connected yet, can not generate packet", out, re.MULTILINE
        )
        assert err == ""
    assert pytest_wrapped_e.type == MeshInterface.MeshInterfaceError


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_fixupPosition_empty_pos():
    """Test _fixupPosition()"""
    iface = MeshInterface(noProto=True)
    pos = {}
    newpos = iface._fixupPosition(pos)
    assert newpos == pos


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_fixupPosition_no_changes_needed():
    """Test _fixupPosition()"""
    iface = MeshInterface(noProto=True)
    pos = {"latitude": 101, "longitude": 102}
    newpos = iface._fixupPosition(pos)
    assert newpos == pos


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_fixupPosition():
    """Test _fixupPosition()"""
    iface = MeshInterface(noProto=True)
    pos = {"latitudeI": 1010000000, "longitudeI": 1020000000}
    newpos = iface._fixupPosition(pos)
    assert newpos == {
        "latitude": 101.0,
        "latitudeI": 1010000000,
        "longitude": 102.0,
        "longitudeI": 1020000000,
    }


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_nodeNumToId(iface_with_nodes):
    """Test _nodeNumToId()"""
    iface = iface_with_nodes
    iface.myInfo.my_node_num = 2475227164
    someid = iface._nodeNumToId(2475227164)
    assert someid == "!9388f81c"


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_nodeNumToId_not_found(iface_with_nodes):
    """Test _nodeNumToId()"""
    iface = iface_with_nodes
    iface.myInfo.my_node_num = 2475227164
    someid = iface._nodeNumToId(123)
    assert someid is None


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_nodeNumToId_to_all(iface_with_nodes):
    """Test _nodeNumToId()"""
    iface = iface_with_nodes
    iface.myInfo.my_node_num = 2475227164
    someid = iface._nodeNumToId(0xFFFFFFFF)
    assert someid == "^all"


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_getOrCreateByNum_minimal(iface_with_nodes):
    """Test _getOrCreateByNum()"""
    iface = iface_with_nodes
    iface.myInfo.my_node_num = 2475227164
    tmp = iface._getOrCreateByNum(123)
    assert tmp == {"num": 123, "user": {"hwModel": "UNSET", "id": "!0000007b", "shortName": "007b", "longName": "Meshtastic 007b"}}


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_getOrCreateByNum_not_found(iface_with_nodes):
    """Test _getOrCreateByNum()"""
    iface = iface_with_nodes
    iface.myInfo.my_node_num = 2475227164
    with pytest.raises(MeshInterface.MeshInterfaceError) as pytest_wrapped_e:
        iface._getOrCreateByNum(0xFFFFFFFF)
    assert pytest_wrapped_e.type == MeshInterface.MeshInterfaceError


@pytest.mark.unit
@pytest.mark.usefixtures("reset_mt_config")
def test_getOrCreateByNum(iface_with_nodes):
    """Test _getOrCreateByNum()"""
    iface = iface_with_nodes
    iface.myInfo.my_node_num = 2475227164
    tmp = iface._getOrCreateByNum(2475227164)
    assert tmp["num"] == 2475227164


# TODO
# @pytest.mark.unit
# def test_enter():
#    """Test __enter__()"""
#    iface = MeshInterface(noProto=True)
#    assert iface == iface.__enter__()


@pytest.mark.unit
def test_exit_with_exception(caplog):
    """Test __exit__()"""
    iface = MeshInterface(noProto=True)
    with caplog.at_level(logging.ERROR):
        iface.__exit__("foo", "bar", "baz")
        assert re.search(
            r"An exception of type foo with value bar has occurred",
            caplog.text,
            re.MULTILINE,
        )
        assert re.search(r"Traceback: baz", caplog.text, re.MULTILINE)


@pytest.mark.unit
def test_showNodes_exclude_self(capsys, caplog, iface_with_nodes):
    """Test that we hit that continue statement"""
    with caplog.at_level(logging.DEBUG):
        iface = iface_with_nodes
        iface.localNode.nodeNum = 2475227164
        iface.showNodes()
        iface.showNodes(includeSelf=False)
        capsys.readouterr()


@pytest.mark.unitslow
def test_waitForConfig(capsys):
    """Test waitForConfig()"""
    iface = MeshInterface(noProto=True)
    # override how long to wait
    iface._timeout = Timeout(0.01)
    with pytest.raises(MeshInterface.MeshInterfaceError) as pytest_wrapped_e:
        iface.waitForConfig()
        assert pytest_wrapped_e.type == MeshInterface.MeshInterfaceError
        out, err = capsys.readouterr()
        assert re.search(
            r"Exception: Timed out waiting for interface config", err, re.MULTILINE
        )
        assert out == ""


@pytest.mark.unit
def test_waitConnected_raises_an_exception(capsys):
    """Test waitConnected()"""
    iface = MeshInterface(noProto=True)
    with pytest.raises(MeshInterface.MeshInterfaceError) as pytest_wrapped_e:
        iface.failure = MeshInterface.MeshInterfaceError("warn about something")
        iface._waitConnected(0.01)
        assert pytest_wrapped_e.type == MeshInterface.MeshInterfaceError
        out, err = capsys.readouterr()
        assert re.search(r"warn about something", err, re.MULTILINE)
        assert out == ""


@pytest.mark.unit
def test_waitConnected_isConnected_timeout(capsys):
    """Test waitConnected()"""
    with pytest.raises(MeshInterface.MeshInterfaceError) as pytest_wrapped_e:
        iface = MeshInterface()
        iface._waitConnected(0.01)
        assert pytest_wrapped_e.type == MeshInterface.MeshInterfaceError
        out, err = capsys.readouterr()
        assert re.search(r"warn about something", err, re.MULTILINE)
        assert out == ""


@pytest.mark.unit
def test_timeago():
    """Test that the _timeago function returns sane values"""
    assert _timeago(0) == "now"
    assert _timeago(1) == "1 sec ago"
    assert _timeago(15) == "15 secs ago"
    assert _timeago(333) == "5 mins ago"
    assert _timeago(99999) == "1 day ago"
    assert _timeago(9999999) == "3 months ago"
    assert _timeago(-999) == "now"

@given(seconds=st.integers())
def test_timeago_fuzz(seconds):
    """Fuzz _timeago to ensure it works with any integer"""
    val = _timeago(seconds)
    assert re.match(r"(now|\d+ (secs?|mins?|hours?|days?|months?|years?))", val)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: meshtastic/protobuf/powermon.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"meshtastic/protobuf/powermon.proto\x12\x13meshtastic.protobuf\"\xe0\x01\n\x08PowerMon\"\xd3\x01\n\x05State\x12\x08\n\x04None\x10\x00\x12\x11\n\rCPU_DeepSleep\x10\x01\x12\x12\n\x0e\x43PU_LightSleep\x10\x02\x12\x0c\n\x08Vext1_On\x10\x04\x12\r\n\tLora_RXOn\x10\x08\x12\r\n\tLora_TXOn\x10\x10\x12\x11\n\rLora_RXActive\x10 \x12\t\n\x05\x42T_On\x10@\x12\x0b\n\x06LED_On\x10\x80\x01\x12\x0e\n\tScreen_On\x10\x80\x02\x12\x13\n\x0eScreen_Drawing\x10\x80\x04\x12\x0c\n\x07Wifi_On\x10\x80\x08\x12\x0f\n\nGPS_Active\x10\x80\x10\"\x88\x03\n\x12PowerStressMessage\x12;\n\x03\x63md\x18\x01 \x01(\x0e\x32..meshtastic.protobuf.PowerStressMessage.Opcode\x12\x13\n\x0bnum_seconds\x18\x02 \x01(\x02\"\x9f\x02\n\x06Opcode\x12\t\n\x05UNSET\x10\x00\x12\x0e\n\nPRINT_INFO\x10\x01\x12\x0f\n\x0b\x46ORCE_QUIET\x10\x02\x12\r\n\tEND_QUIET\x10\x03\x12\r\n\tSCREEN_ON\x10\x10\x12\x0e\n\nSCREEN_OFF\x10\x11\x12\x0c\n\x08\x43PU_IDLE\x10 \x12\x11\n\rCPU_DEEPSLEEP\x10!\x12\x0e\n\nCPU_FULLON\x10\"\x12\n\n\x06LED_ON\x10\x30\x12\x0b\n\x07LED_OFF\x10\x31\x12\x0c\n\x08LORA_OFF\x10@\x12\x0b\n\x07LORA_TX\x10\x41\x12\x0b\n\x07LORA_RX\x10\x42\x12\n\n\x06\x42T_OFF\x10P\x12\t\n\x05\x42T_ON\x10Q\x12\x0c\n\x08WIFI_OFF\x10`\x12\x0b\n\x07WIFI_ON\x10\x61\x12\x0b\n\x07GPS_OFF\x10p\x12\n\n\x06GPS_ON\x10qBc\n\x13\x63om.geeksville.meshB\x0ePowerMonProtosZ\"github.com/meshtastic/go/generated\xaa\x02\x14Meshtastic.Protobufs\xba\x02\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meshtastic.protobuf.powermon_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.geeksville.meshB\016PowerMonProtosZ\"github.com/meshtastic/go/generated\252\002\024Meshtastic.Protobufs\272\002\000'
  _globals['_POWERMON']._serialized_start=60
  _globals['_POWERMON']._serialized_end=284
  _globals['_POWERMON_STATE']._serialized_start=73
  _globals['_POWERMON_STATE']._serialized_end=284
  _globals['_POWERSTRESSMESSAGE']._serialized_start=287
  _globals['_POWERSTRESSMESSAGE']._serialized_end=679
  _globals['_POWERSTRESSMESSAGE_OPCODE']._serialized_start=392
  _globals['_POWERSTRESSMESSAGE_OPCODE']._serialized_end=679
# @@protoc_insertion_point(module_scope)
